/*
*****************************************************************************************************************************************************
Projet Centrale domotique DomoMaison 2017 pour carte Arduino Mega 2560 et carte Ethernet (W5100)
Version 1.70 du 06/09/2017
Copyright 2017 PRIN Jean-Luc
*****************************************************************************************************************************************************
*/
/*
*****************************************************************************************************************************************************
Programme du clavier déporté de la centrale permettant de commander l'alarme Intrusion
*****************************************************************************************************************************************************
*/

// --------------------------------------------------------------------------------------------------------------------------------------------------
// ************************************************* ENTETE DECLARATRICE ****************************************************************************
// --------------------------------------------------------------------------------------------------------------------------------------------------
// Dans cette section, sont déclarées les librairies, les constantes et les variables
// --------------------------------------------- Section librairie et Définitions -------------------------------------------------------------------
// --- Inclusion des librairies  de base nécessaire au système ---
#include <LiquidCrystal.h>          // Librairie pour afficheur LCD
#include <Keypad.h>                 // Librairie pour clavier matriciel
#include <EEPROM.h>                 // librairie pour le stockage de données en mémoire EEPROM interne
#include <SPI.h>                    // Librairie pour la communication via le port SPI
#include <Mirf.h>                   // Librairie pour la gestion de la communication
#include <nRF24L01.h>               // Librairie pour les définitions des registres du nRF24L01
#include <MirfHardwareSpiDriver.h>  // Librairie pour la communication SPI

// --- Déclaration des constantes ---
// #define DEBUG                            // Mode DEBUG pour affichage de informations de test sur le serial port (à mettre systématiquement)
const long SPEED_TRANSMISSION = 57600;              // vitesse pour le port série Degug

//--- Constantes utilisées avec le clavier 4x5
const byte LIGNES = 4;                              // 4 lignes
const byte COLONNES = 5;                            // 5 colonnes
const int DUREE_CLAVIER = 10000;                  	// temps pour appuyer sur une touche, sinon quitte le mode en cours (10000 = 10 sec.)
unsigned long Fin_Saisie;

// --- constantes des broches ---
// Afficheur LCD
const int RS = 7;       // RS de l'afficheur
const int E = 6;        // E de l'afficheur
const int D4 = 5;       // D4 de l'afficheur
const int D5 = 4;       // D5 de l'afficheur
const int D6 = 3;       // D6 de l'afficheur
const int D7 = 2;       // D7 de l'afficheur
const int LCD_EC = 9;   // broche de commande de l'éclairage du LCD
// Clavier 20 touches
const int C5 = 26;      // colonne 5 du clavier
const int C4 = 27;      // colonne 4 du clavier
const int C3 = 28;      // colonne 3 du clavier
const int C2 = 29;      // colonne 2 du clavier
const int C1 = 30;      // colonne 1 du clavier
const int L4 = 25;      // ligne 4 du clavier
const int L3 = 24;      // ligne 3 du clavier
const int L2 = 23;      // ligne 2 du clavier
const int L1 = 22;      // ligne 1 du clavier
const int CLA_EC = 8;   // broche de commande de l'éclairage du clavier
// Durée d'allumage de l'écran en millisecondes
const int DUREE_ECL = 10000;                    // Durée d'allumage de l'écran et du clavier sans intervention = 10 secondes
unsigned long Fin_Eclairage;
boolean Etat_Eclairage;                         // True si afficheur ou clavier allumé
// --- Déclaration des variables globales ---
//--- Définition des touches
char Touches[LIGNES][COLONNES] = {
   {'T','U','V','W','F'},
   {'C','E','O','S','A'},
   {'0','1','2','3','4'},
   {'5','6','7','8','9'}
};

// tableaux de lignes et colonnes
byte BrochesLignes[LIGNES] = {L1, L2, L3, L4};              //connexions utilisées pour les broches de lignes du clavier
byte BrochesColonnes[COLONNES] = {C1, C2, C3, C4, C5};      //connexions utilisées pour les broches de colonnes du clavier

char Touche;                                                // variable de stockage valeur touche appuyée du clavier

// Variables pour Buzzer
const int buzzer = A0;                                      //buzzer sur broche A0

// Variables d'accès
boolean Autorise;
const int DUREE_AUT = 10000;                                // Durée de la validité du code sans intervention = 10 secondes
unsigned long Fin_Autorise;
byte Utilisateur;                                           // Utilisateur qui saisie le code (1 à 4, 1 étant l'administrateur)
byte UtilisateurActif;                                      // Utilisateur en cours d'activité
char CodeU[] = "xxxx";                                      // Code de l'utilisateur

// Variables de l'état de l'alarme
char EtatAlarme;
char EtatZone[3];
char MemEtatAlarme;
char MemEtatZone[3];
boolean DemandeAlarme = false;                              // demande de modification de l'état de l'alarme en cours

// Variables évènements
String EvenementContact = "";
String EvenementPIR = "";
String EvenementSabotage = "";

// Variables pour la transmission NRF24L01
byte Message_Aller[30] = "50;NRFcl;test NRF";
byte Message_Retour[30];
boolean Echec = false;
unsigned long Debut, Seconde, FinAttente, OldSec;
const int DUREE_ATTENTE = 5000;                         // Durée d'attente entre chaque envoi de données (5000 = 5 sec.)
// ------------- Fin de la déclaration des variables globales --------

LiquidCrystal lcd(RS, E, D4, D5, D6, D7);

// création d'un objet keypad = initialisation clavier
 Keypad clavier = Keypad( makeKeymap(Touches), BrochesLignes, BrochesColonnes, LIGNES, COLONNES );
// les broches de lignes sont automatiquement configurées en ENTREE avec pullup interne activé
// les broches de colonnes sont automatiquement configurées en SORTIE

// Initialisation pour la partie débogage
#ifdef DEBUG
    int Compteur_Test = 0;                     // pour TEST
#endif // DEBUG
// --------------------------------------------------------------------------------------------------------------------------------------------------
// ************************************************* DEBUT DU SETUP *********************************************************************************
// --------------------------------------------------------------------------------------------------------------------------------------------------
void setup()
{
  byte i;
    // Pour initialisation les codes d'accès dans l'Eeprom : il faut décommenter la partie suivante (à faire lors du premier démarrage)
    // 1 code admistrateur (le code 1) et 3 codes utilisateurs, tous de 4 chiffres intialisés à 0000
    //    for (i=0; i < 17; i++){
    //        EEPROM.write(i,48);
    //    }

        #ifdef DEBUG
            Serial.begin(SPEED_TRANSMISSION);                                     // initialisation connexion Série pour débogage
            Serial.println(F("Lancement du programme Clavier"));
        #endif

    // Initialisation des broches des LEDs
    pinMode(CLA_EC, OUTPUT);
    pinMode(LCD_EC, OUTPUT);
    pinMode(39, OUTPUT);
    pinMode(40, OUTPUT);
    pinMode(41, OUTPUT);
    pinMode(43, OUTPUT);
    pinMode(44, OUTPUT);
    pinMode(45, OUTPUT);
    pinMode(46, OUTPUT);
    pinMode(45, OUTPUT);
    pinMode(46, OUTPUT);
    digitalWrite(CLA_EC, LOW);
    digitalWrite(LCD_EC, LOW);
    digitalWrite(39, HIGH);
    digitalWrite(40, HIGH);
    digitalWrite(41, HIGH);
    digitalWrite(43, HIGH);
    digitalWrite(44, HIGH);
    digitalWrite(45, HIGH);
    digitalWrite(46, HIGH);

    pinMode(buzzer, OUTPUT);
    lcd.begin(16,2);                                            // Initialise le LCD avec 16 colonnes x 2 lignes
    delay(10);                                                  // pause rapide pour laisser temps initialisation

    digitalWrite(LCD_EC, HIGH);                                 // Eclairage de l'afficheur
    Etat_Eclairage = true;
    lcd.print("DomoMaison 2017");
    lcd.setCursor(0,1);                                         // positionne curseur colonne 1, ligne 2
    lcd.print("Version 1.7");
    delay(3000);                                                // pause de 3 secondes

    // Initialisation du NRF24L01
    // Configuration des broches CSN et CE :
    Mirf.cePin = 48 ;
    Mirf.csnPin = 49 ;
    // configuration du SPI : utiliser le port SPI hardware
    Mirf.spi = &MirfHardwareSpi;
    Mirf.init();                                                // initialisation du module
    // ---------------------------------------------------
    // canal et longueur utile doivent être identiques
    // pour le client et le serveur
    Mirf.channel = 12;
    Mirf.payload = 30;                                           // taille utile de la donnee transmise
    // -------------------------------------------------------
    // Configuration des adresses de reception et d'emission
    Mirf.setRADDR((byte *)"NRFcl");                             // adresse de reception du module (de 5 octets)
    Mirf.setTADDR((byte *)"NRFrl");                             // adresse vers laquelle on transmet (de 5 octets)
    Mirf.configRegister(RF_SETUP,0x26);                         // vitesse = 250Kb/s, Puissance = 0dBm (puissance maximum)
    Mirf.config();                                              // ecriture de la configuration
        #ifdef DEBUG
            Serial.println(F("initialisation NRF OK"));
        #endif
    FinAttente = 0 ;
    lcd.clear();                        // efface écran et met le curseur en haut à gauche
    delay(10);                          // pour laisser temps effacer écran

    // Test de la liaison NRF (ne se fait la première fois au démarrage)
    test_liaison_NRF();

    Touche = NO_KEY;                    // Initialisation de la touche du clavier

    Autorise = false;

} // fin de la fonction setup()
// --------------------------------------------------------------------------------------------------------------------------------------------------
// ************************************************* FIN DU SETUP ***********************************************************************************
// --------------------------------------------------------------------------------------------------------------------------------------------------

// --------------------------------------------------------------------------------------------------------------------------------------------------
// ************************************************* DEBUT DU LOOP **********************************************************************************
// --------------------------------------------------------------------------------------------------------------------------------------------------
void loop()
{
    byte i;
    String StrData;

        if ( millis() > Fin_Eclairage ) {
            digitalWrite(LCD_EC, LOW);                                 // Extinction de l'éclairage de l'afficheur
            digitalWrite(CLA_EC, LOW);                                 // Extinction de l'éclairage du clavier
            Etat_Eclairage = false;
        }

        if (millis() > Fin_Autorise) {
            Autorise = false;
        }

        if (!Autorise && DemandeAlarme) {                           // Si une demande de modif de l'état alarme n'a pas été validée
            DemandeAlarme = false;
                for (i=0; i<3; i++) {
                    EtatZone[i] = MemEtatZone[i];
                }
            affiche_etat_alarme();
        }

        // Si on recoit un message
        if (Mirf.dataReady()) {
            Mirf.getData((byte *) &Message_Retour);
                #ifdef DEBUG
                    Serial.print(F("On a recu : "));
                        for ( i = 0 ; i < Mirf.payload ; i++) {
                            Serial.write(Message_Retour[i]);
                        }
                    Serial.println();
                #endif
            StrData = String((char*)Message_Retour);
                if (StrData.substring(0, 9) == "60;NRFrl;") {            // Message en provenance de la centrale
                    if (StrData.substring(9, 13) == "ETAT") {           // Réception de l'état de fonctionnement de l'alarme
                        charge_etat_alarme(StrData.substring(14, 18));
                        affiche_etat_alarme();
                    }
                    else if (StrData.substring(9, 13) == "DEDE") {
                        digitalWrite(43, LOW);                          // Allumage Led Alarme
                        digitalWrite(46, LOW);                          // Allumage Led PIR
                        EvenementPIR = StrData.substring(14,30);

                    }
                    else if (StrData.substring(9, 13) == "DECO") {
                        digitalWrite(43, LOW);                          // Allumage Led Alarme
                        digitalWrite(45, LOW);                          // Allumage Led Ouverture
                        EvenementContact = StrData.substring(14,30);
                    }
                    envoi_recu();
                }
        }                                                               // sortie de if DataReady

    Touche = clavier.getKey();                                          // lecture de la touche appuyée

        if (Touche != NO_KEY){                                          // si une touche a été frappée -- gestion de la touche appuyée
            delay(300);                                                 // pause entre 2 appuis, evite anti-rebond
                if (!Etat_Eclairage) {
                    digitalWrite(LCD_EC, HIGH);                         // Eclairage de l'afficheur
                    digitalWrite(CLA_EC, HIGH);                         // Eclairage du clavier
                }
            Fin_Eclairage = millis() + DUREE_ECL ;
                if (!Autorise) {
                    saisie_code();
                        if (strncmp(&CodeU[0],"xxxx",4) != 0) {
                            verifie_code();
                                if (Autorise) {
                                    UtilisateurActif = Utilisateur;
                                }
                        }
                }
                else if (Touche == 'C') {
                    change_code();
                }
                else if (Touche == 'A') {
                    if (!DemandeAlarme) {                                           // pas de demande encours
                        DemandeAlarme = true;
                        lcd.clear();
                        affiche_zone();
                            if ((EtatAlarme == '1') || (EtatAlarme == '2')) {       // Alarme activée ou en cours d'activation
                                lcd.print("Demande desacti.");
                            }
                            else {
                                lcd.print("Demande activat.");
                            }
                    }
                    else {
                        envoi_demande();
                    }
                }
                else if ((Touche == 'T') && (DemandeAlarme == true)) {
                    Fin_Eclairage = millis() + DUREE_ECL ;
                    Fin_Autorise = millis() + DUREE_AUT ;
                        if ((EtatZone[0] == '0') || (EtatZone[1] == '0') || (EtatZone[2] == '0')) {
                            for (i=0; i<3; i++) {
                                EtatZone[i] = '1';
                            }
                        }
                        else {
                            for (i=0; i<3; i++) {
                                EtatZone[i] = '0';
                            }
                        }
                    affiche_zone();
                }
                else if ((Touche == 'U') && (DemandeAlarme == true)) {
                    Fin_Eclairage = millis() + DUREE_ECL ;
                    Fin_Autorise = millis() + DUREE_AUT ;
                        if (EtatZone[0] == '0') {
                            EtatZone[0] = '1';
                        }
                        else {
                            EtatZone[0] = '0';
                        }
                    affiche_zone();
                }
                else if ((Touche == 'V') && (DemandeAlarme == true)) {
                    Fin_Eclairage = millis() + DUREE_ECL ;
                    Fin_Autorise = millis() + DUREE_AUT ;
                        if (EtatZone[1] == '0') {
                            EtatZone[1] = '1';
                        }
                        else {
                            EtatZone[1] = '0';
                        }
                    affiche_zone();
                }
                else if ((Touche == 'W') && (DemandeAlarme == true)) {
                    Fin_Eclairage = millis() + DUREE_ECL ;
                    Fin_Autorise = millis() + DUREE_AUT ;
                        if (EtatZone[2] =='0') {
                            EtatZone[2] = '1';
                        }
                        else {
                            EtatZone[2] = '0';
                        }
                    affiche_zone();
                }
                else if(Touche == 'S') {
                    lcd.setCursor(0,1);                             // positionne curseur colonne 1, ligne 2
                        if (EvenementSabotage == "") {
                            lcd.setCursor(0,1);                     // positionne curseur colonne 1, ligne 2
                            lcd.print("Aucun evenement");
                        }
                        else {
                            lcd.print(EvenementSabotage);
                        }
                    delay(5000);
                    lcd.setCursor(0,1);
                    lcd.print("                ");
                    EvenementSabotage = "";
                    digitalWrite(43, HIGH);
                    digitalWrite(44, HIGH);
                }
                else if (Touche == 'O') {
                    lcd.setCursor(0,1);                             // positionne curseur colonne 1, ligne 2
                        if (EvenementContact== "") {
                            lcd.setCursor(0,1);                     // positionne curseur colonne 1, ligne 2
                            lcd.print("Aucun evenement");
                        }
                        else {
                            lcd.print(EvenementContact);
                        }
                    delay(5000);
                    lcd.setCursor(0,1);
                    lcd.print("                ");
                    EvenementContact = "";
                    digitalWrite(43, HIGH);
                    digitalWrite(45, HIGH);
                }
                else if (Touche == 'E') {
                    lcd.setCursor(0,1);                             // positionne curseur colonne 1, ligne 2
                        if (EvenementPIR == "") {
                            lcd.setCursor(0,1);                     // positionne curseur colonne 1, ligne 2
                            lcd.print("Aucun evenement");
                        }
                        else {
                            lcd.print(EvenementPIR);
                        }
                    delay(5000);
                    lcd.setCursor(0,1);
                    lcd.print("                ");
                    EvenementPIR = "";
                    digitalWrite(43, HIGH);
                    digitalWrite(46, HIGH);
                }
        }                                                           // fin gestion de la touche appuyée

	// ---------------------- Partie servant au débogage --------------------------------------------------------------------------------------------
	#ifdef DEBUG
		Compteur_Test ++;
		if (Compteur_Test >= 1000){
			Compteur_Test = 0;
 			//Serial.print(F("Memoire Ram dispo="));
			//Serial.println(freeRam());
		}
    #endif // DEBUG

} // fin de la fonction loop() - le programme recommence au début de la fonction loop sans fin
// --------------------------------------------------------------------------------------------------------------------------------------------------
// ************************************************* DEBUT DU LOOP **********************************************************************************
// --------------------------------------------------------------------------------------------------------------------------------------------------

// --------------------------------------------------------------------------------------------------------------------------------------------------
// ********************************************** DEBUT DES FONCTIONS *******************************************************************************
// --------------------------------------------------------------------------------------------------------------------------------------------------

// ----------------------- début de la fonction test_liaison_NRF (test si la liaison avec la centrale fonctionne) -----------------------------------
void test_liaison_NRF()
{
    byte i;
    byte Nbre = 0;
    String StrData;

    lcd.setCursor(0,0);                             // positionne curseur colonne 1, ligne 1
    lcd.print("Test NRF ");
    lcd.setCursor(0,1);                             // positionne curseur colonne 1, ligne 1
    lcd.print("Temps :");

        while (Nbre < 4) {
            // Envoi du message toutes les 5 secondes
            if ( millis() > FinAttente ) {
                Nbre++;
                    if (Nbre < 4) {
                        lcd.setCursor(9,0);
                        lcd.print(Nbre);
                        lcd.setCursor(8,1);                   // positionne curseur colonne 1, ligne 1
                        lcd.print("  ");
                        envoiMessage();
                    }
            }
            // Si on recoit un message
            if (Mirf.dataReady()) {
                Mirf.getData((byte *) &Message_Retour);
                    #ifdef DEBUG
                        Serial.println(F(" "));
                        Serial.print(F("Retour apres "));
                        Serial.print(millis() - Debut);
                        Serial.println(F(" ms"));
                        Serial.print(F("On a recu : "));
                            for ( i = 0 ; i < Mirf.payload ; i++) {
                                Serial.write(Message_Retour[i]);
                            }
                        Serial.println();
                        Serial.println(F("-----------------------------------"));
                    #endif
                StrData = String((char*)Message_Retour);
                    if (StrData.substring(0, 16) == "60;NRFrl;test OK") {
                        Echec = false ;
                        lcd.clear();
                        lcd.print("Liaison NRF OK");
                        delay(2000);
                    }
            }             // sortie de if DataReady
            else {
                // on va afficher le comptage des secondes
                Seconde = int (( millis() - Debut )) / 1000 ;
                    if (Seconde > OldSec) {
                        #ifdef DEBUG
                            Serial.print (Seconde) ;
                            Serial.print (F(" " )) ;
                        #endif
                        lcd.setCursor(8,1);                   // positionne curseur colonne 1, ligne 1
                        lcd.print(Seconde);
                        OldSec = Seconde ;
                    }       //       sortie du if
            }             //       sortie du else
            if (!Echec) {
                break;
            }
        }
        #ifdef DEBUG
            Serial.println("");
            Serial.println(F("Fin test"));
        #endif

        if (Echec){
            lcd.clear();
            lcd.print("Echec NRF");
            delay(2000);
        }
        else {
            // Demande à la centrale l'état de l'alarme (Initialisation du clavier)
            initialisation();
        }
}
// ----------------------- Fin de la fonction test_liaison_NRF (test si la liaison avec la centrale fonctionne) -------------------------------------

// --------------------------- début de la fonction envoiMessage (envoi les message vers la centrale) -----------------------------------------------
void envoiMessage(void)
{
       if (Echec){
            #ifdef DEBUG
                Serial.println(" ");
                Serial.println(F("------- Time out ------- "));
            #endif
       }
            #ifdef DEBUG
                Serial.println();
                Serial.println(F("Debut envoi"));
            #endif
       // ----------------------------------
       Mirf.send((byte *)&Message_Aller);                           // valeur que l'on veut envoyer
       while(Mirf.isSending()){
             // on attend que le message soit envoye
       }
        #ifdef DEBUG
            Serial.println(F("Envoi Ok"));
            Serial.println(F("Attente retour ..."));
        #endif
       Debut = millis() ;
       FinAttente = Debut + DUREE_ATTENTE;
       OldSec = 0 ;
       Echec = true ;                                   // Echec = True tant que l'on a pas reçu le message de retour
}
// --------------------------- Fin de la fonction envoiMessage (envoi les message vers la centrale) -------------------------------------------------

// --------------------- début de la fonction initialisation (initialise le clavier : état alarme, voyant) ------------------------------------------
void initialisation()
{
    byte i =0;
    byte Nbre = 0;
    String StrData;

    StrData = "50;NRFcl;ETAT";
    StrData.getBytes(Message_Aller, 30);
    FinAttente = 0;
    Nbre = 0;
    lcd.clear();
    lcd.setCursor(0,0);                             // positionne curseur colonne 1, ligne 1
    lcd.print("Demande Init. ");
    lcd.setCursor(0,1);                             // positionne curseur colonne 1, ligne 1
    lcd.print("Temps :");

        while (Nbre < 4) {
            // Envoi du message toutes les 15 secondes
            if ( millis() > FinAttente ) {
                Nbre++;
                    if (Nbre < 4) {
                        lcd.setCursor(14,0);
                        lcd.print(Nbre);
                        lcd.setCursor(8,1);                   // positionne curseur colonne 1, ligne 1
                        lcd.print("  ");
                        envoiMessage();
                    }
            }
            // Si on recoit un message
            if (Mirf.dataReady()) {
                Mirf.getData((byte *) &Message_Retour);
                    #ifdef DEBUG
                        Serial.println(F(" "));
                        Serial.print(F("Retour apres "));
                        Serial.print(millis() - Debut);
                        Serial.println(F(" ms"));
                        Serial.print(F("On a recu : "));
                            for ( i = 0 ; i < Mirf.payload ; i++) {
                                Serial.write(Message_Retour[i]);
                            }
                        Serial.println();
                        Serial.println(F("-----------------------------------"));
                    #endif
                StrData = String((char*)Message_Retour);
                    if (StrData.substring(0, 13) == "60;NRFrl;ETAT") {
                        Echec = false ;
                        lcd.clear();
                        lcd.print("Initiali. OK");
                        delay(2000);
                        charge_etat_alarme(StrData.substring(14,18));
                        affiche_etat_alarme();
                    }
            }             // sortie de if DataReady
            else {
                // on va afficher le comptage des secondes
                Seconde = int (( millis() - Debut )) / 1000 ;
                    if (Seconde > OldSec) {
                        #ifdef DEBUG
                            Serial.print (Seconde) ;
                            Serial.print (F(" " )) ;
                        #endif
                        lcd.setCursor(8,1);                   // positionne curseur colonne 1, ligne 1
                        lcd.print(Seconde);
                        OldSec = Seconde ;
                    }       //       sortie du if
            }             //       sortie du else
            if (!Echec) {
                break;
            }
        }
        if (Echec){
            lcd.clear();
            lcd.print("Echec Init.");
            delay(2000);
       }

}
// --------------------- Fin de la fonction initialisation (initialise le clavier : état alarme, voyanr) --------------------------------------------

// ---------------- début de la fonction charge_etat_alarme (charge les données de l'état de fonctionnement de l'alarme) ----------------------------
void charge_etat_alarme(String Chaine)
{
    byte i;
    char Buf[5];

    Chaine.toCharArray(Buf, 5) ;
    EtatAlarme = Buf[0];
        for(i=0; i<3; i++) {
            EtatZone[i] = Buf[i + 1];
            MemEtatZone[i] = EtatZone[i];
        }

}
// ---------------- Fin de la fonction charge_etat_alarme (charge les données de l'état de fonctionnement de l'alarme) ------------------------------

// ---------------- Début de la fonction affiche_etat_alarme (affiche l'état de fonctionnement de l'alarme) -----------------------------------------
void affiche_etat_alarme()
{
    byte i;

    lcd.clear();
        if (EtatAlarme == '0') {
            lcd.print("Alarme inactive");
        }
        else if (EtatAlarme == '1') {
            lcd.print("Alarme active");
        }
        else if (EtatAlarme == '2') {
            lcd.print("Activa. en cours");
        }
        else if (EtatAlarme == '3') {
            lcd.print("Alarme active S");
        }
        else if (EtatAlarme == '4') {
            lcd.print("Acti. en cours S");
        }

        for(i=0; i<3; i++) {
            if ((EtatZone[i] == '1') && (EtatAlarme != '0')){
                digitalWrite(39 + i, LOW);                                  // Allumage des LED vertes des zones
            }
            else {
                digitalWrite(39 + i, HIGH);                                 // Extinction des LED vertes des zones
            }
        }

    Fin_Eclairage = millis() + DUREE_ECL ;
    digitalWrite(LCD_EC, HIGH);                                             // Eclairage de l'afficheur
    Etat_Eclairage = true;

}
// ---------------- Fin de la fonction affiche_etat_alarme (affiche l'état de fonctionnement de l'alarme) -------------------------------------------

// ---------------- Début de la fonction affiche_zone (affiche l'état des zones, même si l'alarme n'est pas activé) ---------------------------------
void affiche_zone()
{
    byte i;

        for(i=0; i<3; i++) {
            if (EtatZone[i] == '1'){
                digitalWrite(39 + i, LOW);                                 // Allumage des LED vertes des zones
            }
            else {
                digitalWrite(39 + i, HIGH);                                 // Extinction des LED vertes des zones
            }
        }
}
// ---------------- Fin de la fonction affiche_zone (affiche l'état des zones, même si l'alarme n'est pas activé) -----------------------------------

// ----------------- Début de la fonction envoi_demande (envoi la demande d'activation ou de désactivation de l'alarme) -----------------------------
void envoi_demande()
{
    byte i =0;
    byte Nbre = 0;
    String StrData;

    lcd.clear();
    lcd.print("Dem. en cours :");

    StrData = "50;NRFcl;METAT;";
    StrData.getBytes(Message_Aller, 30);
    MemEtatAlarme =  EtatAlarme;
        if ((EtatAlarme == '1') || (EtatAlarme == '2')) {
            EtatAlarme = '0';
        }
        else {
            EtatAlarme = '1';
        }
    Message_Aller[15] = EtatAlarme;
        for (i=0; i<3; i++) {
            Message_Aller[16 + (i * 2)] = ';';
            Message_Aller[17 + (i * 2)] = EtatZone[i];
        }
    StrData = String((char*)Message_Aller);
    Serial.println(StrData);
    FinAttente = 0;
    Nbre = 0;
    lcd.setCursor(0,1);                             // positionne curseur colonne 1, ligne 1
    lcd.print("Temps :");

        while (Nbre < 4) {
            // Envoi du message toutes les 15 secondes
            if ( millis() > FinAttente ) {
                Nbre++;
                    if (Nbre < 4) {
                        lcd.setCursor(15,0);
                        lcd.print(Nbre);
                        lcd.setCursor(8,1);                   // positionne curseur colonne 1, ligne 1
                        lcd.print("  ");
                        envoiMessage();
                    }
            }
            // Si on recoit un message
            if (Mirf.dataReady()) {
                Mirf.getData((byte *) &Message_Retour);
                    #ifdef DEBUG
                        Serial.println(F(" "));
                        Serial.print(F("Retour apres "));
                        Serial.print(millis() - Debut);
                        Serial.println(F(" ms"));
                        Serial.print(F("On a recu : "));
                            for ( i = 0 ; i < Mirf.payload ; i++) {
                                Serial.write(Message_Retour[i]);
                            }
                        Serial.println();
                        Serial.println(F("-----------------------------------"));
                    #endif
                StrData = String((char*)Message_Retour);
                    if (StrData.substring(0, 13) == "60;NRFrl;ETAT") {
                        Echec = false ;
                        charge_etat_alarme(StrData.substring(14,18));
                        affiche_etat_alarme();
                    }
            }             // sortie de if DataReady
            else {
                // on va afficher le comptage des secondes
                Seconde = int (( millis() - Debut )) / 1000 ;
                    if (Seconde > OldSec) {
                        #ifdef DEBUG
                            Serial.print (Seconde) ;
                            Serial.print (F(" " )) ;
                        #endif
                        lcd.setCursor(8,1);                   // positionne curseur colonne 1, ligne 1
                        lcd.print(Seconde);
                        OldSec = Seconde ;
                    }       //       sortie du if
            }             //       sortie du else
            if (!Echec) {
                break;
            }
        }
        if (Echec){
            lcd.clear();
            lcd.print("Echec Demande");
            EtatAlarme = MemEtatAlarme;
                for (i=0; i<3; i++) {
                    EtatZone[i] = MemEtatZone[i];
                }
            delay(2000);
       }
}
// ----------------- Fin de la fonction envoi_demande (envoi la demande d'activation ou de désactivation de l'alarme) -------------------------------

// ----------------- Début de la fonction envoi_recu (envoi de la confirmation de la réception d'un message) ----------------------------------------
void envoi_recu()
{
    String StrData;

    StrData = "50;NRFcl;RECU";
    StrData.getBytes(Message_Aller, 30);

        #ifdef DEBUG
            Serial.println();
            Serial.println(F("Envoi recu"));
        #endif
       // ----------------------------------
       Mirf.send((byte *)&Message_Aller);                           // valeur que l'on veut envoyer
       while(Mirf.isSending()){
             // on attend que le message soit envoye
       }
        #ifdef DEBUG
            Serial.println(F("Envoi Ok"));
        #endif
}
// ----------------- Fin de la fonction envoi_recu (envoi de la confirmation de la réception d'un message) ----------------------------------------

// ---------------------- Début de la fonction saisie_code (permet de saisir le code d'accès à l'alarme) --------------------------------------------
void saisie_code()
{
    byte Nbre = 0;
    byte i;

    lcd.setCursor(0,1);                                                         // positionne curseur colonne 1, ligne 1
    lcd.print("Code U  :       ");
    lcd.setCursor(7,1);

    Fin_Saisie = millis() + DUREE_CLAVIER ;

        while (Nbre < 5) {
            Touche = clavier.getKey();                                          // lecture de la touche appuyée
                if (Touche != NO_KEY){                                          // si une touche a été frappée -- gestion de la touche appuyée
                    delay(300);                                                 // pause entre 2 appuis, evite anti-rebond
                    Fin_Saisie = millis() + DUREE_CLAVIER ;
                    Fin_Eclairage = millis() + DUREE_ECL ;
                        if (Nbre == 0) {
                            if ((Touche < 49) || (Touche > 52)) {
                                son_erreur();
                            }
                            else {
                                Utilisateur = Touche - 48;
                                lcd.setCursor(6,1);                                                         // positionne curseur colonne 1, ligne 1
                                lcd.print(Utilisateur);
                                lcd.setCursor(10,1);                                                         // positionne curseur colonne 1, ligne 1
                                Nbre++;
                            }
                        }
                        else {
                            if ((Touche < 48) || (Touche > 57)) {
                                son_erreur();
                            }
                            else {
                                CodeU[Nbre - 1] = Touche;
                                lcd.print(Touche);
                                Nbre++;
                            }
                        }
                }
            if (millis() > Fin_Saisie) {
                lcd.setCursor(0,1);                                             // positionne curseur colonne 1, ligne 1
                lcd.print("                ");
                    for (i=0; i<4; i++) {
                        CodeU[i] = 'x';
                    }
                break;
            }
        }
}
// ---------------------- Fin de la fonction saisie_code (permet de saisir le code d'accès à l'alarme) ----------------------------------------------

// ---------------------- Début de la fonction verifie_code (Compare le code en mémoire et celui saisie) --------------------------------------------
void verifie_code()
{
    byte i;
    char CodeMem []= "0000";
    byte T;

    // Charge le code en EEPROM correspondant à l'utilisateur
    Utilisateur = Utilisateur - 1;
        for (i=0; i<4; i++) {
            CodeMem[i] = EEPROM.read(i + (Utilisateur * 4));
            T = EEPROM.read(i + (Utilisateur * 4));
        }
    lcd.setCursor(0,1);                                                 // positionne curseur colonne 1, ligne 1
        if (strncmp(&CodeMem[0], &CodeU[0],4) == 0) {
            lcd.print("Code bon        ");
            son_ok();
            Autorise = true;
            Fin_Autorise = millis() + DUREE_AUT ;

        }
        else {
            lcd.print("Code faux       ");
            son_erreur();
        }
}
// ---------------------- Fin de la fonction verifie_code (Compare le code en mémoire et celui saisie) ----------------------------------------------

// ---------------------- Début de la fonction son_erreur (emet ce son lors d'une erreur ou code erroné) --------------------------------------------
void son_erreur()
{
    tone(buzzer, 1000);                                 // Send 1KHz sound signal...
    delay(1000);                                        // ...for 1 sec
    noTone(buzzer);                                     // Stop sound...
}
// ---------------------- Fin de la fonction son_erreur (emet ce son lors d'une erreur ou code erroné) ----------------------------------------------

// ---------------------- Début de la fonction son_ok (emet ce son lors d'une saisie valide) --------------------------------------------------------
void son_ok()
{
    tone(buzzer, 1000);                                 // Send 1KHz sound signal...
    delay(200);                                        // ...for 1 sec
    noTone(buzzer);                                     // Stop sound...
    delay(200);                                        // ...for 1 sec
    tone(buzzer, 1000);                                 // Send 1KHz sound signal...
    delay(200);                                        // ...for 1 sec
    noTone(buzzer);                                     // Stop sound...
}
// ---------------------- Fin de la fonction son_ok (emet ce son lors d'une saisie valide) ----------------------------------------------------------

// ---------------------- Début de la fonction change_code (permet de modifier le code d'accès) -----------------------------------------------------
void change_code()
{
    char Code1[] = "xxxx";
    char Code2[] = "xxxx";
    byte Utilisateur1;
    byte Utilisateur2;
    byte i;
    byte Temp;

    lcd.clear();                                             // positionne curseur colonne 1, ligne 1
    lcd.print("Nouveau code ?");
    saisie_code();
        for (i=0; i<4; i++) {
            Code1[i] = CodeU[i];
        }
    Utilisateur1 = Utilisateur - 1;

        if (Code1[0] != 'x') {
                if ((UtilisateurActif != 0) && (Utilisateur1 != UtilisateurActif)) {     // Interdit de changer le code d'un autre utilisateur, sauf l'administrateur (1)
                    lcd.clear();
                    lcd.print("Non autorise");
                    son_erreur();
                    delay(1000);
                }
                else {
                    lcd.clear();                                                        // positionne curseur colonne 1, ligne 1
                    lcd.print("Confirmez :");
                    saisie_code();
                        for (i=0; i<4; i++) {
                            Code2[i] = CodeU[i];
                        }
                    Utilisateur2 = Utilisateur - 1;
                    lcd.clear();
                        if ((Utilisateur1 == Utilisateur2) && (Code1[0] == Code2[0]) && (Code1[1] == Code2[1]) && (Code1[2] == Code2[2]) && (Code1[3] == Code2[3])) {
                                for (i=0; i<4; i++) {
                                    EEPROM.write(i + (Utilisateur1 * 4), Code1[i]);
                                }
                            lcd.print("Code valide");
                            son_ok();
                            delay(1000);
                        }
                        else {
                            lcd.print("Code invalide");
                            son_erreur();
                            delay(1000);
                        }
                }
        }

    affiche_etat_alarme();

}
// ---------------------- Fin de la fonction change_code (permet de modifier le code d'accès) -------------------------------------------------------

// -------------------------- Début de la fonction freeRam : Calcul mémoire ram dynamique -----------------------------------------------------------
int freeRam ()
{
  extern int __heap_start, *__brkval;
  int v;
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
}
// ------------------- fin de la fonction freeRam ---------------------------------------------------------------------------------------------------
